from typing import Final

from ..utils.cert import DaneCert
from ..utils.tlsa.updater import TlsaUpdater
from .base import HandlerBase

HELP_UPDATETLSA_DRYRUN: Final = """\
Don't actually update TLSA records, just output what the operation \
would do.\
"""


class UpdateTlsaHandler(HandlerBase):
    'Class for listing adoptable certificates.'

    def _get_adopted_certs(self) -> dict[str, DaneCert]:
        'Return the `DaneCert`s to update TLSA records on.'
        result = dict[str, DaneCert]()
        if not self._args.hostnames:
            for hostname, conf_file in self.all_certs.lineages.items():
                if conf_file not in self.all_certs.already_adopted:
                    continue
                result[conf_file] = self.all_certs.parsed[conf_file]
            return result
        for hostname in self._args.hostnames:
            conf_file = self.all_certs.lineages.get(hostname)
            if not conf_file:
                raise KeyError(
                    f'Hostname (lineage) {hostname!r} is not unknown by ' +
                    'certbot.')
            if conf_file not in self.all_certs.already_adopted:
                raise KeyError(
                    f'Hostname (lineage) {hostname!r} is not yet adopted. ' +
                    'Please use the adopt command first.')
            result[conf_file] = self.all_certs.parsed[conf_file]
        return result

    def _process(self) -> int:
        adopted_certs = self._get_adopted_certs()
        for conf_file, cert in adopted_certs.items():
            updater = TlsaUpdater(
                verbose=self._args.verbose, dry_run=self._args.dry_run,
                config=self._config, cert=cert, certconfig_path=conf_file)
            updater.process()
        return 0
