'See `config.sample.yaml` for the depicted types.'

from typing import Literal

from typing_extensions import TypedDict

from .common import LITERAL_TCP_OR_UDP, LITERAL_TSIG_ALGORITHMS


class DefaultsTlsaPortDict(TypedDict):
    '`defaults["tlsa"]["records"]` section of the raw configuration.'
    protocol: LITERAL_TCP_OR_UDP
    portnumber: int


class DefaultTlsaRrtypeDict(TypedDict):
    """
    One `defaults["tlsa"]["rrtypes"]` section item of the raw
    configuration.
    """
    usage: int
    selector: int
    matching_type: int


class DefaultsTlsaDict(TypedDict):
    '`defaults["tlsa"]` section of the raw configuration.'
    records: list[DefaultsTlsaPortDict]
    ttl: int
    time_to_reload: int
    rrtypes: list[DefaultTlsaRrtypeDict]


class DefaultsDict(TypedDict):
    '`defaults` section of the raw configuration.'
    tlsa: DefaultsTlsaDict
    method: Literal['dns-update']


class DnsUpdateServerTsigDict(TypedDict):
    '`dns-update["servers"]["tsig"]` section item of the raw configuration.'
    name: str
    algorithm: LITERAL_TSIG_ALGORITHMS
    secret: str


class DnsUpdateServerDict(TypedDict):
    '`dns-update["servers"]` section item of the raw configuration.'
    ip: str
    tsig: DnsUpdateServerTsigDict


class DnsUpdateDict(TypedDict):
    '`dns-update` section of the raw configuration.'
    servers: list[DnsUpdateServerDict]


class PathsNsupdateDict(TypedDict):
    '`paths["nsupdate"]` section of the raw configuration.'
    executable: str
    params: list[str]


class PathsDict(TypedDict):
    '`paths` section of the raw configuration.'
    runtimeyaml: str
    nsupdate: PathsNsupdateDict


ConfDict = TypedDict('ConfDict', {
    'defaults': DefaultsDict,
    'dns-update': DnsUpdateDict,
    'paths': PathsDict
})
