from daneupdate.utils.cert import DaneCert

from ..config import Configuration
from .liveupdate_config import HostnameConfig


class TlsaUpdater(object):
    'Updating TLSA records for an deployed and/or upcoming certificate.'

    def __init__(
        self, verbose: bool, dry_run: bool, config: Configuration,
        cert: DaneCert, certconfig_path: str
    ):
        self._verbose = verbose
        self._dry_run = dry_run
        self._adopted_config = config.runtime.adopted[certconfig_path]
        self._config = config
        self._cert = cert

    def _get_hostname_configs(self) -> list[HostnameConfig]:
        """
        Return the configured `AdoptedCertHostitemConfig`s or construct
        one.
        """
        if self._adopted_config.hosts:
            return HostnameConfig.from_adoptedhosts(
                hosts=self._adopted_config.hosts, config=self._config,
                cert=self._cert)
        return [
            HostnameConfig.from_hostname(
                hostname=hostname, config=self._config)
            for hostname in self._cert.common_names]

    def process(self) -> None:
        """
        Search for existing records and update them to the required
        state.
        """
        if self._config.defaults.method == 'dns-update':
            for hostname_config in self._get_hostname_configs():
                hostname_records = \
                    hostname_config.get_tlsa_records(cert=self._cert)
                for server in hostname_config.servers:
                    server.update(
                        hostname_records=hostname_records,
                        verbose=self._verbose, dry_run=self._dry_run)
            return
        raise NotImplementedError(
            f'DNS update method {self._config.defaults.method!r} not ' +
            'implemented (yet).')
